function [Atilde ttilde] = affineCompoundShapeAlignment(L_t, L_o)
%  This is the sample implementation of the binary image registration
%  algorithm described in the following paper:
%
%    Csaba Domokos and Zoltan Kato.
%    Affine Shape Alignment Using Covariant Gaussian Densities: A Direct
%    Solution.
%    Journal of Mathematical Imaging and Vision, 2014. (Accepted)
%
%  Please acknowledge the use of our program by referring to the
%  publications above.
%
%  Input:
%    L_t, L_o: template and observation are labelled images consisting of
%              m >= 2components
%  Output:
%    Atilde, tilde: estimated affine transformation
%
%  Author : Csaba Domokos (c.domokos.res@gmail.com)
%           https://sites.google.com/site/cdomokosres/
%  Date   : 25.10.2014
%           12.12.2008

Atilde = zeros(2);
ttilde = zeros(2, 1);

% Check the input
m_t = max(L_t(:));
assert(m_t >= 2, 'The template image must contain at least two objects')
assert(m_t == max(L_o(:)), 'The Template and Observation images should contain the same number of objects')
m = m_t;

% Calculate normalized pixel coordinates of L_t, L_o calculated as in
% Eq. (16)
[P_t N_t t_t] = normalization(L_t);
[P_o N_o t_o] = normalization(L_o);

% Choose a set of exponents {n_i in R | i=1...\ell}
w = [1 3 5 1/3 1/5];

% Initialize r in [1, 3]
r = 2;

% Solving Eq. (15): C_o (resp. C_t ) denotes the coefficient matrix (resp.
% constants) on the left (resp. right) hand side of the system of
% equations
C_o = Coeffs([P_o double(reshape(L_o(L_o > 0), [], 1))], m, r, w, 3);
C_t = Coeffs([P_t double(reshape(L_t(L_t > 0), [], 1))], m, r, w, 2);
C_t = sqrt(det(cov(P_o)) / det(cov(P_t))) * C_t;

S = pinv(C_o) * C_t;
Astar = inv(S(1 : 2, 1 : 2)');
tstar = -Astar * S(3, 1 : 2)';

% Unnormalization
invN_o = inv(N_o);
Atilde = invN_o * Astar * N_t;
ttilde = invN_o * (Astar * t_t + tstar - t_o);

function [P N t] = normalization(L)
% NORMALIZATION calculates the normalized pixel coordinates
%  Input:
%    L: original labelled image
%  Output:
%    P: normalized pixel coordinates
%    N, t: normalization transformation

[y x] = find(L);
t = -mean([x y])';
P = bsxfun(@plus, [x y], t');  % translataion

N = diag(1 ./ max(abs(P)));
t = N * t;
P = P * N';  % scaling

function C = Coeffs(P, m, r, w, c)
% COEFFS computes coefficient matrices C_o and C_t for Algorithm 1.
%  Input:
%    P: labeled pixel coordinates
%    m: number of components
%    r: parameter r
%    w: the set of exponents
%    c: number of columns of the matrix C
%  Output:
%    C: coefficient matrix

% Set all element of C to 0
l = numel(w);  % number of exponents
C = zeros(l, c);

% Compute mu and Sigma over P
%mu = mean(P); % null vector due to the normalization
invSigma = inv(cov(P(:, 1 : 2)));

for i = 1 : m  % integrating of each pdf over the domain of the main pdf
    % Compute mu_i and Sigma_i of the ith component
    Idx = find(P(:, 3) == i);  % coordinates of the ith object
    mu_i = mean(P(Idx, 1 : 2))';
    invSigma_i = inv(cov(P(Idx, 1 : 2)));

    % Diagonalize Sigma_i^{-1}
    % Compute G_i and D_i based on Eq. (18)
    invG_i = [1 -invSigma_i(1, 2) / invSigma_i(1, 1); 0 1];
    d11 = invSigma_i(1, 1);
    d22 = det(invSigma_i) / invSigma_i(1, 1);
    
    invSigmahat = invG_i' * invSigma * invG_i;
    muhat = -inv(invG_i) * mu_i;
    
    % Computing the coefficients
    z_1_min = muhat(1) - r * sqrt(invSigmahat(2, 2) / det(invSigmahat));
    z_1_max = muhat(1) + r * sqrt(invSigmahat(2, 2) / det(invSigmahat));
    z_2_min = muhat(2) - r * sqrt(invSigmahat(1, 1) / det(invSigmahat));
    z_2_max = muhat(2) + r * sqrt(invSigmahat(1, 1) / det(invSigmahat));
    for j = 1 : l
        n = w(j);
        
        C_1 = sqrt(n * d22 / 2);
        C_2 = sqrt(n * d11 / 2);
        Integrals = [1 / C_1 * quadgk(@(z_1) z_1 .* fun(n, d11, z_1, C_1, @(z_1) u_minmax(z_1, muhat, invSigmahat, r)), z_1_min, z_1_max),...
                     1 / C_2 * quadgk(@(z_2) z_2 .* fun(n, d22, z_2, C_2, @(z_2) v_minmax(z_2, muhat, invSigmahat, r)), z_2_min, z_2_max),...
                     1 / C_2 * quadgk(@(z_2) fun(n, d22, z_2, C_2, @(z_2) v_minmax(z_2, muhat, invSigmahat, r)), z_2_min, z_2_max)];
        if c == 3
            C(j, :) = C(j, :) + Integrals * [1, 0, 0; invG_i(1, 2), 1, 0; -muhat(1) - invG_i(1, 2) * muhat(2), -muhat(2), 1];
        else
            C(j, :) = C(j, :) + Integrals * [1, 0; invG_i(1, 2), 1; -muhat(1) - invG_i(1, 2) * muhat(2), -muhat(2)];
        end
    end
end
C = sqrt(pi) / 2 * C;

% Auxiliary functions for calculating the integrals defined in Eq. (20)-(22)
function val = u_minmax(z_1, muhat, invSigmahat, r)
z_1 = z_1 - muhat(1);
w_1 = sqrt(invSigmahat(2, 2) * r ^ 2 - det(invSigmahat) * z_1 .^ 2);
val = muhat(2) - (invSigmahat(1, 2) * repmat(z_1, [2 1]) + [w_1; -w_1]) / invSigmahat(2, 2);

function val = v_minmax(z_2, muhat, invSigmahat, r)
z_2 = z_2 - muhat(2);
w_2 = sqrt(invSigmahat(1, 1) * r ^ 2 - det(invSigmahat) * z_2 .^ 2);
val = muhat(1) - (invSigmahat(1, 2) * repmat(z_2, [2 1]) + [w_2; -w_2]) / invSigmahat(1, 1);

function val = fun(n, d, z, C, minmaxFun)
minmaxFunVal = minmaxFun(z);
val = exp(-n * d * z .^ 2 / 2) .* (erf(C * minmaxFunVal(2, :)) - erf(C * minmaxFunVal(1, :)));