function [delta diffImg] = calcDelta(Template, Observation, Atilde, ttilde)
% CALCDELTA computes the absolute difference between the Observation and
%   the Registered image (calculated by applying the affine transformation
%   (Atilde, ttilde) to the Template image)
%
%  Input:
%    Template, Observation: template and observation images
%    Atilde, ttilde: estimated affine transformation
%  Output:
%    delta: absolute difference between the Observation and the Registered
%           images
%    diffImg: RGB image that shows the absolute difference of the images.
%             The overlapping areas are shown by yellow. Note that this
%             image is cropped based on the bounding box of the Observation
%             and Registered shapes. (This image is calculated when
%             delta < 1
%    BB: coordinates of the topleft and bottomright corners of diffImg
%
%  Author : Csaba Domokos (c.domokos.res@gmail.com)
%           https://sites.google.com/site/cdomokosres/
%  Date   : 25.10.2014
%           12.12.2008

assert(ndims(Template) == 2, 'Template image must be a single channel image')
assert(ndims(Observation) == 2, 'Observation image must be a single channel image')

assert(all(size(Atilde) == [2 2]), 'Atilde must be a 2 x 2 matrix')
assert(all(size(ttilde) == [2 1]), 'tilde must be a 2 x 1 vector')

delta = NaN;
diffImg = [];

templBB = BoundingBox(Template);
obsBB = BoundingBox(Observation);

% check whether the bounding boxes of the Registered and Observation image
% have any overlap
tform = maketform('affine', [Atilde, ttilde]');
outbounds = findbounds(tform, templBB);
if isempty(OverlapOfBoundingBoxes(outbounds, obsBB)) % There is no overlapping area
    delta = 100;
    return
end

% calculate the registered image
try
    [Registered XData YData] = imtransform(Template(templBB(1, 2) : templBB(2, 2), templBB(1, 1) : templBB(2, 1)),...
                                           tform, 'nearest',...
                                           'UData', templBB(:, 1)', 'VData', templBB(:, 2)');
catch
    error('The delta error measure has not been calculated.\n')
end
regCorners = round([XData' YData']);
regBB = BoundingBox(Registered);
Registered = Registered(regBB(1, 2) : regBB(2, 2), regBB(1, 1) : regBB(2, 1));
regCorners = bsxfun(@plus, regBB, regCorners(1, :) - 1);

% delta error measure
ROI = OverlapOfBoundingBoxes(obsBB, regCorners);
if isempty(ROI)
    delta = 100;
else
    obsROI = OverlapOfBoundingBoxes(obsBB, ROI);
    regROI = bsxfun(@minus, OverlapOfBoundingBoxes(regCorners, ROI), regCorners(1, :) - 1);

    delta = numel(find(Observation(obsROI(1, 2) : obsROI(2, 2), obsROI(1, 1) : obsROI(2, 1)))) +...
            numel(find(Registered(regROI(1, 2) : regROI(2, 2), regROI(1, 1) : regROI(2, 1)))) -...
            numel(find(xor(Observation(obsROI(1, 2) : obsROI(2, 2), obsROI(1, 1) : obsROI(2, 1)), Registered(regROI(1, 2) : regROI(2, 2), regROI(1, 1) : regROI(2, 1)))));
    delta = (1 - double(delta) / (numel(find(Observation(obsBB(1, 2) : obsBB(2, 2), obsBB(1, 1) : obsBB(2, 1)))) + numel(find(Registered(regROI(1, 2) : regROI(2, 2), regROI(1, 1) : regROI(2, 1)))))) * 100;
end

% calculate the absolute difference image
if nargout > 1
    BB = [min([obsBB(1, :); regCorners(1, :)]); max([obsBB(2, :); regCorners(2, :)])];
    try        
        diffImg = zeros([BB(2, 2) - BB(1, 2) + 1, BB(2, 1) - BB(1, 1) + 1, 3], 'uint8');
    catch
        warning('The difference image has not been calculated.\n')
        return
    end
   
    % red channel
    ROI = OverlapOfBoundingBoxes(regCorners, BB);
    diffImgROI = bsxfun(@minus, OverlapOfBoundingBoxes(BB, ROI), BB(1, :) - 1);
    regROI = bsxfun(@minus, OverlapOfBoundingBoxes(regCorners, ROI), regCorners(1, :) - 1);
    diffImg(diffImgROI(1, 2) : diffImgROI(2, 2), diffImgROI(1, 1) : diffImgROI(2, 1), 1) = 255 * Registered(regROI(1, 2) : regROI(2, 2), regROI(1, 1) : regROI(2, 1));
    
    % green channel
    ROI = OverlapOfBoundingBoxes(obsBB, BB);
    diffImgROI = bsxfun(@minus, OverlapOfBoundingBoxes(BB, ROI), BB(1, :) - 1);
    obsROI = OverlapOfBoundingBoxes(obsBB, ROI);
    diffImg(diffImgROI(1, 2) : diffImgROI(2, 2), diffImgROI(1, 1) : diffImgROI(2, 1), 2) = 255 * Observation(obsROI(1, 2) : obsROI(2, 2), obsROI(1, 1) : obsROI(2, 1));
end    

function BB = BoundingBox(I)
% BOUNDINGBOX calculates the bounbding box of the object in the image I.

[y x] = find(I);
BB = [x(1) min(y); x(end) max(y)];

function BB = OverlapOfBoundingBoxes(BB1, BB2)
% OVERLAPOFBUNDINGBOXES calculates the coordinates of the topleft and
%   bottomtright corners of the overlapping area of the bounding boxes BB1
%   and BB2

BB = [max([BB1(1, :); BB2(1, :)]); min([BB1(2, :); BB2(2, :)])];
if any(BB(2, :) - BB(1, :) < 0)
    BB = [];
end