function [delta absDiffImg] = diffImage(P1, P2, R)
% DIFFIMAGE compute the difference between two image by an affine
%     transformation. [delta DifferenceWithColor] = diffImage(P1, P2, R),
%     where P1, P2 are two 2D input image and R (3x3) is the affine
%     transformation between them. The transformation is applied to the
%     first image and then delta, is the absolute difference, will be
%     computed. The images are placed into a common coordinate system too.
%     The resulted DifferenceImage is an RGB image, where the Red channel
%     contains the first image, the transformed image is on the Green
%     channel, so the yellow means the overlayed area.
%
%    Example 1
%    ---------
%
%    Template = bwlabel(im2bw(imread('model.png')));
%    Observation = bwlabel(im2bw(imread('trans.png')));
%
%    Reg = mpdf_numeric_list(Template, Observation);
%    [delta DifferenceImage] = diffImage(Template, Observation, Reg);
%
%    ---------
%    Project's homepage: http://www.inf.u-szeged.hu/~dcs/projects/mpdf
%
%    Author : Csaba Domokos (dcs@inf.u-szeged.hu)
%             http://www.inf.u-szeged.hu/~dcs
%    Date   : 12.12.2008
warning off all
delta = NaN;
absDiffImg = [];

Template = P1;
Observation = P2;
%Template = im2bw(P1, 0);
%Observation = im2bw(P2, 0);

[templateHeight templateWidth] = size(Template);

% checking the bounding boxes
[y x] = find(Observation);
W = R * [1 templateWidth templateWidth 1; 1 1 templateHeight templateHeight; 1 1 1 1];
W = W(1 : 2, :)';
Bound = round([min(W); max(W)]);
if max(x) < Bound(1, 1) || max(y) < Bound(1, 2) || Bound(2, 1) < min(x) || Bound(2, 2) < min(y) % No overlapping area
    delta = 100;
    %fprintf(1, 'Error=100.000000\n');
    return
end

try
    Registred = imtransform(Template,...
                            maketform('affine', R(1 : 2, 1 : 3)'), 'nearest',...
                            'XData', Bound(:, 1)', 'YData', Bound(:, 2)',...
                            'Size', [Bound(2, 2) - Bound(1, 2) + 1, Bound(2, 1) - Bound(1, 1) + 1]);
catch
    delta = -1;
    warning('Error measure has not been evaluated!\n')
    return
end
[observationHeight observationWidth] = size(Observation);
observationPixel = numel(find(Observation(:)));

registredPixel = numel(find(Registred(:)));

minX = max(1, Bound(1, 1));
minY = max(1, Bound(1, 2));
maxX = min(observationWidth, Bound(2, 1));
maxY = min(observationHeight, Bound(2, 2));

RIdxMinX = -Bound(1, 1) + 1 + minX;
RIdxMaxX = -Bound(1, 1) + 1 + maxX;
RIdxMinY = -Bound(1, 2) + 1 + minY;
RIdxMaxY = -Bound(1, 2) + 1 + maxY;

% error measure: delta
delta = observationPixel - numel(find(Observation(minY : maxY, minX : maxX)));
delta = delta + registredPixel - numel(find(Registred(RIdxMinY : RIdxMaxY, RIdxMinX : RIdxMaxX)));
delta = delta + numel(find(xor(Observation(minY : maxY, minX : maxX), Registred(RIdxMinY : RIdxMaxY, RIdxMinX : RIdxMaxX))));
delta = delta / (observationPixel + registredPixel) * 100;

if nargout > 1
    try
        DifferenceWithColor = zeros([max(Bound(2, 2), observationHeight) - min(Bound(1, 2), 1) + 1, max(Bound(2, 1), observationWidth) - min(Bound(1, 1), 1) + 1, 3]);
        ImgBound = [min(1, Bound(1, 1)), min(1, Bound(1, 2)); max(observationWidth, Bound(2, 1)) max(observationHeight, Bound(2, 2))];

        DifferenceWithColor(size(DifferenceWithColor, 1) - (ImgBound(2, 2) - Bound(1, 2)) : size(DifferenceWithColor, 1) - (ImgBound(2, 2) - Bound(2, 2)), ...
            size(DifferenceWithColor, 2) - (ImgBound(2, 1) - Bound(1, 1)) : size(DifferenceWithColor, 2) - (ImgBound(2, 1) - Bound(2, 1)), 1) = Registred;


        DifferenceWithColor(size(DifferenceWithColor, 1) - (ImgBound(2, 2) - 1) : size(DifferenceWithColor, 1) - (ImgBound(2, 2) - observationHeight), ...
            size(DifferenceWithColor, 2) - (ImgBound(2, 1) - 1) : size(DifferenceWithColor, 2) - (ImgBound(2, 1) - observationWidth), 2) = Observation;

    catch
        warning('Difference image has not been calculated!\n')
    end
end